import random
import time


def request(client, config, tag, get_by_instance_id, open_port):
    request_id = _perform_request(client, config)
    time.sleep(2)

    request = _describe_request(client, request_id)
    if request.status_code == 'price-too-low':
        raise RuntimeError(request.status_message)

    _wait_until_completed(client, request_id)
    request = _describe_request(client, request_id)

    tag(client, request.instance_id, config)
    _wait_until_running(client, request.instance_id)

    instance = get_by_instance_id(client, request.instance_id)
    if instance.has_security_groups:
        open_port(client, instance, 8888)
    return instance


def _perform_request(client, config):
    random_id = str(random.random() * 1000)
    response = client.request_spot_instances(
        SpotPrice=config.max_bid,
        ClientToken=random_id,
        InstanceCount=1,
        Type='one-time',
        LaunchSpecification={
            'ImageId': config.ami,
            'KeyName': config.key_name,
            'InstanceType': config.type,
            'Placement': {
                'AvailabilityZone': config.az,
            }
        }
    )
    return response.get('SpotInstanceRequests')[0].get('SpotInstanceRequestId')


def _wait_until_completed(client, request_id):
    waiter = client.get_waiter('spot_instance_request_fulfilled')
    return waiter.wait(SpotInstanceRequestIds=[request_id])


def _describe_request(client, request_id):
    response = client.describe_spot_instance_requests(
        SpotInstanceRequestIds=[request_id],
    )
    return SpotInstanceRequest(response.get('SpotInstanceRequests')[0])


def _wait_until_running(client, instance_id):
    waiter = client.get_waiter('instance_running')
    return waiter.wait(InstanceIds=[instance_id])

class SpotInstanceRequest():
    def __init__(this, response):
        this.instance_id = response.get('InstanceId')
        this.status_code = response.get('Status').get('Code')
        this.status_message = response.get('Status').get('Message')



import boto3
import time
from .client import build as build_client
from .instance import find_latest as find_latest_instance
from .instance import tag
from .config import Config
from .spin_cursor import spin


def snapshot(args):
    client = build_client(args)
    conf = Config(client, args)
    instance = find_latest_instance(client, conf)

    with spin(">> Creating snapshot for instance: " + str(instance.ip_address)):
        snap = create_and_wait(client, instance, conf)

    return snap

def create_and_wait(client, instance, conf):
    image = create(client, instance)
    tag(client, image.id, conf)
    wait_for_completion(client, image)
    return image


def create(client, instance):
    now = time.strftime("%Y-%m-%d %H-%M")
    response = client.create_image(
        Name=("Spotr image {0}".format(now)),
        Description="Spotr image",
        InstanceId=instance.id)
    return Snapshot(response)


def wait_for_completion(client, image):
    waiter = client.get_waiter('image_available')
    waiter.wait(
        Filters=[
            {
                'Name': 'image-id',
                'Values': [image.id]
            },
        ]
    )

class Snapshot():
    def __init__(this, response):
        this.id = response['ImageId']
